﻿using System.Threading.Tasks;
using DeukBot4.MessageHandlers.CommandHandler.RequestStructure;
using DeukBot4.MessageHandlers.Permissions;
using Discord;
using Discord.WebSocket;

namespace DeukBot4.MessageHandlers.CommandHandler
{
    public class ModeratorCommands : CommandContainerBase
    {
        public override string Name => "Moderator";

        [Command("kick", PermissionLevel.Moderator)]
        [CommandParameters(ParameterMatcher.ParameterType.User, ParameterMatcher.ParameterType.Remainder)]
        [CommandHelp("Kicks a user from the server",
            "Kicks a user from the server. Will not work on people with a helper role, or higher.\n" +
            "Usage: \n" +
            "``!kick {User Mention} {optional: Reason}``\n" +
            "``!kick {User ID} {optional: Reason}``")]
        [BlockUsageInPm, RequireParameterMatch]
        public async Task KickUser(CommandRequest request)
        {
            // get the server channel object out of message. Return if it's somehow not a server channel
            if (!(request.OriginalMessage.Channel is IGuildChannel channel))
                return;

            // get the id of the user, this parses the string to an id
            var user = await request.Parameters[0].AsDiscordUser(channel.Guild);

            // get the permissions of the user we want to kick
            var userPermissions =
                await PermissionValidator.GetUserPermissionLevel(request.OriginalMessage.Channel, (SocketUser) user);
            // if the user has sufficient permissions, or is deukbot, warn the user that he's not allowed to do that, and stop
            if (userPermissions >= PermissionLevel.Helper || user.Id == Program.Client.CurrentUser.Id)
            {
                await request.SendMessageAsync("You are not allowed to kick that user");
                return;
            }

            // see if a reason was given, use that if so
            var reason = string.Empty;
            if (request.Parameters.Length >= 2)
            {
                reason = request.Parameters[1].AsString();
            }

            // and kick
            await user.KickAsync(reason);
            await request.SendMessageAsync($"User was kicked: {user.Username}");
        }

        [Command("ban", PermissionLevel.Moderator)]
        public async Task BanUser(CommandRequest request)
        {
            // get the server channel object out of message. Return if it's somehow not a server channel
            if (!(request.OriginalMessage.Channel is IGuildChannel channel))
                return;

            // get the id of the user, this parses the string to an id
            var user = await request.Parameters[0].AsDiscordUser(channel.Guild);

            // get the permissions of the user we want to kick
            var userPermissions =
                await PermissionValidator.GetUserPermissionLevel(request.OriginalMessage.Channel, (SocketUser) user);
            // if the user has sufficient permissions, or is deukbot, warn the user that he's not allowed to do that, and stop
            if (userPermissions >= PermissionLevel.Helper || user.Id == Program.Client.CurrentUser.Id)
            {
                await request.SendMessageAsync("You are not allowed to ban that user");
                return;
            }

            // see if a reason was given, use that if so
            var reason = string.Empty;
            if (request.Parameters.Length >= 2)
            {
                reason = request.Parameters[1].AsString();
            }

            // and ban
            await channel.Guild.AddBanAsync(user, 0, reason);
            await request.SendMessageAsync($"User was banned: {user.Username}");
        }
    }
}